from mllm.utils.maps import parallel_map, p_map
import time
def test_parallel_map():
    def wait_for_1_second(x):
        time.sleep(0.2)
        return x**2

    for i, res in parallel_map(wait_for_1_second, [1, 2, 3], n_workers=1, title="test"):
        print(res)
def test_p_map():
    def wait_for_1_second(x):
        time.sleep(0.2)
        return x**2

    for arg, res in p_map(wait_for_1_second, [1, 2, 3], n_workers=1, title="test"):
        print(arg, res)
        assert arg**2 == res



import os

import dotenv


def update_env_var(env_var_name, env_var_value, verbose):
    try:
        dotenv_file = dotenv.find_dotenv(raise_error_if_not_found=True)
        print("Updating .env file at " + dotenv_file + "? (y/n)")
        choice = input()
        if choice != "y":
            print("No .env file updated. Exiting.")
    except IOError:
        print("No .env file found.")
        print("Creating a new .env file at user home directory? (y/n)")
        choice = input()
        if choice == "y":
            dotenv_file = os.environ.get("HOME") + "/.env"
        else:
            print("No .env file created. Exiting.")
            return
    dotenv.set_key(dotenv_file, env_var_name, env_var_value)
    if verbose:
        print(f"Environment variable {env_var_name} has been updated to {env_var_value}.")
        print(f"env file: {dotenv_file} has been updated.")
    dotenv.load_dotenv(dotenv_file, override=True)
    return



if __name__ == '__main__':
    update_env_var("MY_VAR", "MY_VALUE1", True)
    print(os.environ.get("MY_VAR"))


from mllm import Chat


def test_image_chat():
    chat = Chat()
    chat.add_user_message("What is in this image?")
    chat.add_image_message(
        "EvoLogoOrg.png")
    res = chat.complete()
    print(res)

def test_image_chat_cache():
    chat = Chat()
    chat += "What is in this image?"
    chat.add_image_message("EvoLogoOrg.png")
    res = chat.complete(cache=True)
    print(res)

def test_image_chat_dict():
    chat = Chat()
    chat.add_user_message("What is in this image? Answer in JSON")
    chat.add_image_message(
        "EvoLogoOrg.png")
    res = chat.complete(parse="dict", model="gpt-4o")
    print(res)


from mllm.chat import Chat
from mllm.embedding import get_embeddings


def test_basic():
    inputs = ["a", "b", "c"]
    res = get_embeddings(inputs)
    print(res)


def test_chat():
    chat = Chat()
    chat.add_user_message("Who is your developer?")
    res = chat.complete()
    chat += "Can you give more information about it?"
    res = chat.complete()
    print(chat)


def test_system_message():
    chat = Chat(system_message="You are an assistant who only reply in `yes` or `no`")
    chat += "Are you a human?"
    res = chat.complete()
    print(res)

def test_auto_dedent():
    chat = Chat()
    chat += """
    Explain the following code
    ```python
    def greet():
        print("Hello")
    greet()
    ```"""
    chat += "What is the output?"
    message = chat.get_messages_to_api()
    assert message[0]["content"] == """Explain the following code
```python
def greet():
    print("Hello")
greet()
```
What is the output?"""




def get_llava_response(messages, options, model_name):
    image, prompt = extract_prompt_and_image(messages)
    try:
        import replicate
    except ImportError:
        raise ImportError("Please install the `replicate` package to use the llava model")
    model_name = model_name.split("/")[1:]
    model_name = "/".join(model_name)
    output = replicate.run(
        model_name,
        input={
            "image": image,
            "top_p": 1,
            "prompt": prompt,
            "history": [],
            "max_tokens": 2048,
            "temperature": options["temperature"],
        }
    )

    res = "".join(output)
    return res


def extract_prompt_and_image(messages):
    prompt = []
    image = None
    for message in messages:
        if message["role"] == "user":
            for item in message["content"]:
                if item["type"] == "text":
                    prompt.append(item["text"])
                elif item["type"] == "image_url":
                    image = item["image_url"]["url"]
        elif message["role"] == "system":
            prompt.append("<system>" + message["content"] + "</system>")
    prompt = "\n".join(prompt)
    return image, prompt


special_models = {
    "replicate/yorickvp/llava-v1.6-34b:41ecfbfb261e6c1adf3ad896c9066ca98346996d7c4045c5bc944a79d430f174": get_llava_response
}

def get_special_model_handler(model_name):
    handler = special_models.get(model_name, None)
    if handler is not None:
        return lambda messages, options: handler(messages, options, model_name)
    return None







from mllm import Chat
from mllm.provider_switch import set_default_to_llama, set_default_to_anthropic, \
    set_default_to_gemini

def test_gemini():
    set_default_to_gemini()
    run_chat()

def test_image_gemini():
    set_default_to_gemini()
    run_image_chat()

def test_anthropic():
    set_default_to_anthropic()
    run_chat()

def test_image_anthropic():
    set_default_to_anthropic()
    run_image_chat()

def test_llama():
    set_default_to_llama()
    run_chat()

def test_image_llava():
    set_default_to_llama()
    run_image_chat()

def run_chat():
    chat = Chat()
    chat += "What is your name"
    chat += "Response by a json"
    res = chat.complete(parse="dict")
    print(res)

def run_image_chat():
    chat = Chat()
    chat += "What is in this image?"
    chat.add_image_message("EvoLogoOrg.png")
    res = chat.complete(cache=False)
    print(res)